@@ -19,6 +19,7 @@ import shlex
 import json
 import logging
 import base64
+from pathlib import Path
 
 from . import __version__
 from .params import KeeperParams
@@ -28,9 +29,26 @@ from . import cli
 
 def get_params_from_config(config_filename):
     params = KeeperParams()
-    params.config_filename = (
-        config_filename or os.getenv('KEEPER_CONFIG_FILE', 'config.json')
-    )
+
+    if os.getenv("KEEPER_COMMANDER_DEBUG"):
+        logging.getLogger().setLevel(logging.DEBUG)
+        logging.info('Debug ON')
+
+    opts, flags = parser.parse_known_args(sys.argv[1:])
+
+    if os.getenv('KEEPER_CONFIG_FILE'):
+        logging.debug("Setting config file from KEEPER_CONFIG_FILE env variable %s" % os.getenv('KEEPER_CONFIG_FILE'))
+        params.config_filename = os.getenv('KEEPER_CONFIG_FILE')
+    elif opts.launched_with_shortcut:
+        launcher_keeper_folder_path = Path.home().joinpath('.keeper')
+        launcher_keeper_folder_path.mkdir(parents=True, exist_ok=True)
+
+        # unix: /Users/username/.keeper/
+        # win: C:\\Users\\username\\.keeper\\
+
+        params.config_filename = str(launcher_keeper_folder_path.joinpath('config.json'))
+    else:
+        params.config_filename = config_filename or 'config.json'
 
     if os.path.exists(params.config_filename):
         try:
@@ -69,7 +87,7 @@ def get_params_from_config(config_filename):
                             logging.info('Debug ON')
 
                     if 'batch_mode' in params.config:
-                        params.batch_mode = params.config['batch_mode'] == True
+                        params.batch_mode = params.config['batch_mode'] is True
 
                     if 'device_id' in params.config:
                         device_id = base64.urlsafe_b64decode(params.config['device_id'] + '==')
@@ -78,12 +96,17 @@ def get_params_from_config(config_filename):
                     if 'logout_timer' in params.config:
                         params.logout_timer = params.config['logout_timer']
 
-                    if 'login_v3' in params.config:
-                        params.login_v3 = params.config['login_v3']
-
                     if 'private_key' in params.config:
                         params.device_private_key = params.config['private_key']
 
+                    if 'proxy' in params.config:
+                        params.proxy = params.config['proxy']
+
+                    if 'certificate_check' in params.config:
+                        check = params.config['certificate_check']
+                        if isinstance(check, bool):
+                            params.rest_context.certificate_check = check
+
             except Exception as e:
                 logging.error('Unable to parse JSON configuration file "%s"', params.config_filename)
                 answer = input('Do you want to delete it (y/N): ')
@@ -95,7 +118,7 @@ def get_params_from_config(config_filename):
             logging.warning('Error: Unable to open config file %s: %s', params.config_filename, ioe)
 
     if not params.server:
-        params.server = 'https://keepersecurity.com/api/v2/'
+        params.server = 'keepersecurity.com'
 
     return params
 
@@ -115,7 +138,10 @@ parser.add_argument('--version', dest='version', action='store_true', help='Disp
 parser.add_argument('--config', dest='config', action='store', help='Config file to use')
 parser.add_argument('--debug', dest='debug', action='store_true', help='Turn on debug mode')
 parser.add_argument('--batch-mode', dest='batch_mode', action='store_true', help='Run commander in batch or basic UI mode.')
-parser.add_argument('--login-v3', '-lv3', dest='login_v3', action='store', help='Use Login v3 to login to Keeper.')
+parser.add_argument('--launched-with-shortcut', '-lwsc', dest='launched_with_shortcut', action='store',
+                    help='Indicates that the app was launched using a shortcut, for example using Mac App or from '
+                         'Windows Start Menu.')
+parser.add_argument('--proxy', dest='proxy', action='store', help='Proxy server..')
 parser.add_argument('command', nargs='?', type=str, action='store', help='Command')
 parser.add_argument('options', nargs='*', action='store', help='Options')
 parser.error = usage
@@ -129,6 +155,9 @@ def handle_exceptions(exc_type, exc_value, exc_traceback):
 
 
 def main(from_package=False):
+
+    set_working_dir()
+
     errno = 0
 
     if from_package:
@@ -139,21 +168,19 @@ def main(from_package=False):
     opts, flags = parser.parse_known_args(sys.argv[1:])
     params = get_params_from_config(opts.config)
 
-    if opts.debug:
-        params.debug = opts.debug
-        is_debug = logging.getLogger().level <= logging.DEBUG
-        logging.getLogger().setLevel((logging.WARNING if params.batch_mode else logging.INFO) if is_debug else logging.DEBUG)
-        logging.info('Debug %s', 'OFF' if is_debug else 'ON')
-
-
     if opts.batch_mode:
         params.batch_mode = True
 
-    if opts.login_v3:
-        params.login_v3 = 'TRUE'.startswith(str(opts.login_v3).upper())
+    if opts.debug:
+        params.debug = opts.debug
+
+    logging.basicConfig(level=logging.WARNING if params.batch_mode else logging.DEBUG if opts.debug else logging.INFO, format='%(message)s')
+
+    if opts.proxy:
+        params.proxy = opts.proxy
 
     if opts.server:
-        params.server = 'https://{0}/api/v2/'.format(opts.server)
+        params.server = opts.server
 
     if opts.user:
         params.user = opts.user
@@ -181,29 +208,41 @@ def main(from_package=False):
         if opts.command == '?' or not params.commands:
             usage('')
 
-    logging.basicConfig(level=logging.WARNING if params.batch_mode else logging.INFO, format='%(message)s')
-
     if params.timedelay >= 1 and params.commands:
         cli.runcommands(params)
     else:
-        if opts.command not in {'shell', '-'}:
+        if opts.command in {'shell', '-'}:
+            if opts.command == '-':
+                params.batch_mode = True
+        elif opts.command and os.path.isfile(opts.command):
+            with open(opts.command, 'r') as f:
+                lines = f.readlines()
+                params.commands.extend([x.strip() for x in lines])
+            params.commands.append('q')
+            params.batch_mode = True
+        else:
+            flags = ' '.join([shlex.quote(x) for x in flags]) if flags is not None else ''
+            options = ' '.join([shlex.quote(x) for x in opts.options]) if opts.options is not None else ''
             if opts.command:
-                flags = ' '.join([shlex.quote(x) for x in flags]) if flags is not None else ''
-                options = ' '.join([shlex.quote(x) for x in opts.options]) if opts.options is not None else ''
-                command = ' '.join([opts.command, flags])
+                command = ' '.join([opts.command or '', flags])
                 if options:
                     command += ' -- ' + options
                 params.commands.append(command)
             params.commands.append('q')
             params.batch_mode = True
-        else:
-            if opts.command == '-':
-                params.batch_mode = True
 
         errno = cli.loop(params)
 
     sys.exit(errno)
 
 
+def set_working_dir():
+
+    opts, flags = parser.parse_known_args(sys.argv[1:])
+
+    if opts.launched_with_shortcut:
+        os.chdir(Path.home())
+
+
 if __name__ == '__main__':
     main()
