@@ -5,35 +5,39 @@
 #              |_|            
 #
 # Keeper Commander 
-# Copyright 2017 Keeper Security Inc.
+# Copyright 2021 Keeper Security Inc.
 # Contact: ops@keepersecurity.com
 #
 
-import os
-import sys
-import getpass
 import datetime
-import time
-import threading
 import functools
 import logging
+import os
 import re
-
+import shlex
+import subprocess
+import sys
+import threading
+import time
 from collections import OrderedDict
 
 from prompt_toolkit import PromptSession
-from prompt_toolkit.shortcuts import CompleteStyle
 from prompt_toolkit.enums import EditingMode
+from prompt_toolkit.shortcuts import CompleteStyle
 
+from . import api, display, loginv3, ttk
+from .commands import (
+    register_commands, register_enterprise_commands, register_msp_commands,
+    aliases, commands, command_info, enterprise_commands, msp_commands
+)
 from .commands.msp import get_mc_by_name_or_id
-
-from .params import KeeperParams
-from . import display, loginv3, api
-# from .api import sync_down, login, communicate, query_enterprise, query_msp, login_and_get_mc_params, login_and_get_mc_params_login_v3
+from .constants import OS_WHICH_CMD, KEEPER_PUBLIC_HOSTS
 from .error import AuthenticationError, CommunicationError, CommandError
+from .params import KeeperParams
+from .recordv3 import init_recordv3_commands
 from .subfolder import BaseFolderNode
 from .autocomplete import CommandCompleter
-from .commands import register_commands, register_enterprise_commands, register_msp_commands, aliases, commands, enterprise_commands, msp_commands
+from . import versioning
 
 
 stack = []
@@ -129,10 +133,7 @@ def check_if_running_as_mc(params, args):
         # Running commands as Managed Company admin via MSP
 
         if current_mc_id not in mc_params_dict:
-            if params.login_v3:
-                mc_params = api.login_and_get_mc_params_login_v3(params, current_mc_id)
-            else:
-                mc_params = api.login_and_get_mc_params(params, current_mc_id)
+            mc_params = api.login_and_get_mc_params_login_v3(params, current_mc_id)
             mc_params_dict[current_mc_id] = mc_params
 
         params = mc_params_dict[current_mc_id]
@@ -173,7 +174,42 @@ def do_command(params, command_line):
         display.formatted_history(stack)
         return
 
-    if '-h' in command_line.lower():
+    if command_line.lower().startswith('server'):
+        _, sp, server = command_line.partition(' ')
+        if server:
+            if not params.session_token:
+                server = server.strip()
+                region = next((x for x in KEEPER_PUBLIC_HOSTS.items()
+                               if server.casefold() in [x[0].casefold(), x[1].casefold()]), None)
+                if region:
+                    params.server = region[1]
+                    logging.info('Keeper region is set to %s', region[0])
+                else:
+                    params.server = server
+                    logging.info('Keeper server is set to %s',  params.server)
+            else:
+                logging.warning('Cannot change Keeper region while logged in')
+        else:
+            print(params.server)
+        return
+
+    if command_line.startswith('ksm'):
+        try:
+            subprocess.check_call([OS_WHICH_CMD, 'ksm'], stdout=subprocess.DEVNULL)
+        except subprocess.CalledProcessError:
+            logging.error(
+                'Please install the ksm application to run ksm commands.\n'
+                'See https://docs.keeper.io/secrets-manager/secrets-manager'
+                '/secrets-manager-command-line-interface'
+                '#secrets-manager-cli-installation'
+            )
+        else:
+            if sys.platform.startswith('win'):
+                subprocess.check_call(command_line)
+            else:
+                subprocess.check_call(shlex.split(command_line))
+        return
+    elif '-h' in command_line.lower():
         if command_line.lower().startswith('h ') or command_line.lower().startswith('history '):
             print("usage: history|h [-h]")
             print("\nShow command history.")
@@ -277,7 +313,6 @@ def do_command(params, command_line):
         current_mc_id = None
 
         api.query_enterprise(params)
-        api.query_msp(params)
 
     else:
         cmd, args = command_and_args_from_cmd(command_line)
@@ -307,7 +342,6 @@ def do_command(params, command_line):
                 if command.is_authorised():
                     if not params.session_token:
                         try:
-                            prompt_for_credentials(params)
                             api.login(params)
                             api.sync_down(params)
                         except KeyboardInterrupt as e:
@@ -346,13 +380,23 @@ def do_command(params, command_line):
                 display_command_help(show_enterprise=(params.enterprise is not None))
 
 
-def runcommands(params):
+def runcommands(params, commands=None, command_delay=0, quiet=False):
+    if commands is None:
+        commands = params.commands
+
     keep_running = True
+    first_command = True
     timedelay = params.timedelay
 
     while keep_running:
-        for command in params.commands:
-            logging.info('Executing [%s]...', command)
+        for command in commands:
+            if first_command:
+                first_command = False
+            elif command_delay != 0:
+                time.sleep(command_delay)
+
+            if not quiet:
+                logging.info('Executing [%s]...', command)
             try:
                 do_command(params, command)
             except CommunicationError as e:
@@ -373,28 +417,6 @@ def runcommands(params):
                 keep_running = False
 
 
-def prompt_for_username_if_needed(params):
-    if not params.user:
-        params.user = getpass.getpass(prompt='User(Email): ', stream=None)
-
-        while not params.user:
-            params.user = getpass.getpass(prompt='User(Email): ', stream=None)
-
-
-def prompt_for_credentials(params):
-
-    if not params.login_v3:
-        prompt_for_username_if_needed(params)
-
-        while not params.password:
-            try:
-                params.password = getpass.getpass(prompt='Password: ', stream=None)
-            except KeyboardInterrupt:
-                print('')
-            except EOFError:
-                return 0
-
-
 def force_quit():
     try:
         if os.name == 'posix':
@@ -411,8 +433,6 @@ prompt_session = None
 
 
 def loop(params):  # type: (KeeperParams) -> int
-    logging.debug('Params: %s', params)
-
     global prompt_session
     error_no = 0
     suppress_errno = False
@@ -429,29 +449,37 @@ def loop(params):  # type: (KeeperParams) -> int
                                            complete_while_typing=False)
 
         display.welcome()
+        versioning.welcome_print_version(params)
+
     else:
         logging.getLogger().setLevel(logging.WARNING)
 
     if params.user:
-        if len(params.commands) == 0:
-            if not params.login_v3 and not params.password:
-                logging.info('Enter password for {0}'.format(params.user))
-                try:
-                    if not params.login_v3:
-                        params.password = getpass.getpass(prompt='Password: ', stream=None)
-                except KeyboardInterrupt:
-                    print('')
-                except EOFError:
-                    return 0
-    # if params.password:
         try:
             api.login(params)
             if params.session_token:
                 do_command(params, 'sync-down')
+        except KeyboardInterrupt:
+            print('')
+        except EOFError:
+            return 0
         except Exception as e:
             logging.error(e)
 
+        # add ability to manipulate w/ legacy or v3 records
+        # determined by the response from the server
+        init_recordv3_commands(params)
+    else:
+        if params.server:
+            logging.info('Current Keeper region: %s', params.server)
+        logging.info('Use "server" command to change Keeper region > "server US"')
+        for region in KEEPER_PUBLIC_HOSTS:
+            logging.info('\t%s: %s', region, KEEPER_PUBLIC_HOSTS[region])
+
     while True:
+        if params.session_token:
+            ttk.TTK.update(params)
+
         command = ''
         if len(params.commands) > 0:
             command = params.commands[0].strip()
@@ -486,11 +514,13 @@ def loop(params):  # type: (KeeperParams) -> int
             if command.startswith("@"):
                 suppress_errno = True
                 command = command[1:]
+            if params.batch_mode:
+                logging.info('> %s', command)
             error_no = 1
             result = do_command(params, command)
             error_no = 0
             if result:
-                logging.warning(result)
+                print(result)
         except EOFError:
             break
         except KeyboardInterrupt:
@@ -501,13 +531,7 @@ def loop(params):  # type: (KeeperParams) -> int
             else:
                 logging.warning('%s', e.message)
         except CommunicationError as e:
-
-            if e.result_code == 'restricted_client_type':
-
-                logging.error("Error code: %s\n%s" % e.result_code, loginv3.permissions_error_msg)
-            else:
-                logging.error("Error code: %s\nCommunication Error: %s" % (e.result_code, e.message))
-
+            logging.error("Communication Error: %s", e.message)
         except AuthenticationError as e:
             logging.error("AuthenticationError Error: %s", e.message)
         except Exception as e:
