@@ -10,18 +10,19 @@
 #
 
 import argparse
-import datetime
 import logging
 import re
 
 from tabulate import tabulate
 
+from keepercommander.commands import recordv3
 from ..params import KeeperParams
 from .. import api
 from ..commands.base import raise_parse_exception, suppress_exit, Command
 from . import plugin_manager
 from .. import generator
 from ..subfolder import find_folders, get_folder_path
+from ..utils import confirm
 
 
 def register_commands(commands):
@@ -37,6 +38,8 @@ def register_command_info(aliases, command_info):
 rotate_parser = argparse.ArgumentParser(prog='rotate|r', description='Rotate the password for a Keeper record from this Commander.')
 rotate_parser.add_argument('--print', dest='print', action='store_true', help='display the record content after rotation')
 rotate_parser.add_argument('--match', dest='match', action='store', help='regular expression to select records for password rotation')
+rotate_parser.add_argument('--password', dest='password', action='store', help='new password (optional)')
+rotate_parser.add_argument('--force', dest='force', action='store_true', help='force all matches to rotate without prompt')
 rotate_parser.add_argument('name', nargs='?', type=str, action='store', help='record UID or name assigned to rotate command')
 rotate_parser.error = raise_parse_exception
 rotate_parser.exit = suppress_exit
@@ -52,19 +55,54 @@ def adjust_password(password):   # type: (str) -> str
             return password[i] + password[1:i] + password[0] + password[i+1:]
     return 'a' + password
 
-def rotate_password(params, record_uid, name=None):
+
+def get_v2_or_v3_custom_field_value(record, custom_field_name, default_value=None):
+    if record.record_type:  # V3 record
+        matches = [x for x in record.custom_fields if x.get('name', x.get('label')).endswith(custom_field_name)]
+        if len(matches) > 0:
+            value_list = matches[0].get('value') or default_value
+            if isinstance(value_list, list):
+                ret_val = value_list[0] if len(value_list) > 0 else default_value
+            else:
+                ret_val = value_list
+        else:
+            ret_val = default_value
+
+    else:  # V2 record
+        matches = [x for x in record.custom_fields if x['name'] == custom_field_name]
+        ret_val = matches[0]['value'] if len(matches) > 0 else default_value
+
+    return ret_val
+
+
+def update_v2_or_v3_password(params, record, new_password):
+    if record.record_type:  # V3 record
+        return_result = {}
+        recordv3.RecordEditCommand().execute(
+            params, command='edit', password=new_password, record=record.record_uid, return_result=return_result
+        )
+        return return_result.get('update_record_v3', False)
+
+    else:  # V2 record
+        record.password = new_password
+        return api.update_record(params, record)
+
+
+def rotate_password(params, record_uid, name=None, new_password=None):
     """ Rotate the password for the specified record """
     api.sync_down(params)
     record = api.get_record(params, record_uid)
 
     # execute rotation plugin associated with this record
     plugin_name = None
-    plugins = [x for x in record.custom_fields if x['name'].startswith('cmdr:plugin')]
+    plugins = [x for x in record.custom_fields if 'cmdr:plugin' in x.get('name', x.get('label', ''))]
     if plugins:
         if name:
-            plugins = [x for x in plugins if x['name'] == 'cmdr:plugin:' + name]
+            plugins = [x for x in plugins if x.get('name', x.get('label')).endswith('cmdr:plugin:' + name)]
     if plugins:
         plugin_name = plugins[0]['value']
+    if isinstance(plugin_name, list):
+        plugin_name = plugin_name[0]
     if not plugin_name:
         logging.error('Record is not marked for password rotation (i.e. \'cmdr:plugin\' custom field).\n'
                       'Add custom field \'cmdr:plugin\'=\'noop\' to enable password rotation for this record')
@@ -74,22 +112,23 @@ def rotate_password(params, record_uid, name=None):
     if not plugin:
         return False
 
-    # generate a new password with any specified rules
-    rules = record.get("cmdr:rules")
-    if rules:
-        logging.debug("Rules found for record")
-        new_password = generator.generateFromRules(rules)
-    else:
-        logging.debug("No rules, just generate")
-        new_password = generator.generate()
+    if not new_password:
+        # generate a new password with any specified rules
+        rules = get_v2_or_v3_custom_field_value(record, "cmdr:rules")
+        if rules:
+            logging.debug("Rules found for record")
+            new_password = generator.generateFromRules(rules)
+        else:
+            logging.debug("No rules, just generate")
+            new_password = generator.generate()
 
-    # ensure password starts with alpha numeric character
-    new_password = adjust_password(new_password)
+        # ensure password starts with alpha numeric character
+        new_password = adjust_password(new_password)
 
-    # Some plugins might need to change the password in the process of rotation
-    # f.e. windows plugin gets rid of certain characters.
-    if hasattr(plugin, "adjust"):
-        new_password = plugin.adjust(new_password)
+        # Some plugins might need to change the password in the process of rotation
+        # f.e. windows plugin gets rid of certain characters.
+        if hasattr(plugin, "adjust"):
+            new_password = plugin.adjust(new_password)
 
     # log_message = 'Rotated on {0}'.format(datetime.datetime.now().ctime())
     # if record.notes:
@@ -111,7 +150,7 @@ def rotate_password(params, record_uid, name=None):
         logging.warning("Password rotation failed for record uid=[%s], plugin \"%s\"." % (record.record_uid, plugin_name))
         return False
 
-    if api.update_record(params, record):
+    if update_v2_or_v3_password(params, record, new_password):
         new_record = api.get_record(params, record_uid)
         logging.info('Rotation successful for record_uid=%s, revision=%d', new_record.record_uid, new_record.revision)
         return True
@@ -141,6 +180,7 @@ class RecordRotateCommand(Command):
         print_result = kwargs['print'] if 'print' in kwargs else None
         name = kwargs['name'] if 'name' in kwargs else None
         match = kwargs['match'] if 'match' in kwargs else None
+        force = kwargs['force'] if 'force' in kwargs else None
         if name:
             record_uid = None
             rotate_name = None
@@ -148,7 +188,10 @@ class RecordRotateCommand(Command):
                 record_uid = name
             else:
                 RecordRotateCommand.find_endpoints(params)
-                endpoints = [x for x in RecordRotateCommand.Endpoints if x.name.lower() == name.lower()]
+                nl = name.lower()
+                endpoints = [
+                    x for x in RecordRotateCommand.Endpoints if x.record_title.lower() == nl or x.name.lower() == nl
+                ]
                 if len(endpoints) > 0:
                     if len(endpoints) == 1:
                         record_uid = endpoints[0].record_uid
@@ -157,7 +200,7 @@ class RecordRotateCommand(Command):
                         logging.error('There are more than one rotation records with name %s. Please use record UID.', name)
                         return
             if record_uid:
-                rotate_password(params, record_uid, name=rotate_name)
+                rotate_password(params, record_uid, name=rotate_name, new_password=kwargs.get('password'))
                 if print_result:
                     record = api.get_record(params, record_uid)
                     record.display()
@@ -166,10 +209,11 @@ class RecordRotateCommand(Command):
         elif match:
             results = api.search_records(params, match)
             for r in results:
-                rotate_password(params, r.record_uid)
-                if print_result:
-                    record = api.get_record(params, r.record_uid)
-                    record.display()
+                if force or confirm(f'Rotate password for record {r.title}?'):
+                    rotate_password(params, r.record_uid, new_password=kwargs.get('password'))
+                    if print_result:
+                        record = api.get_record(params, r.record_uid)
+                        record.display()
         else:
             RecordRotateCommand.find_endpoints(params)
             if RecordRotateCommand.Endpoints:
@@ -202,8 +246,8 @@ class RecordRotateCommand(Command):
                     endpoints = {}    # type: {str, str}
                     endpoints_desc = {}
                     for field in record.custom_fields:
-                        if 'name' in field:
-                            field_name = field['name']
+                        field_name = field.get('name', field.get('label'))
+                        if field_name:
                             m = rotate_pattern.match(field_name)
                             if m and 'value' in field:
                                 endpoints[field_name] = field['value']
