@@ -7,15 +7,17 @@
 # Keeper Commander 
 # Contact: ops@keepersecurity.com
 #
+
 import datetime
 import hashlib
-import base64
 import hmac
+import itertools
 
 from urllib import parse
 
-from .subfolder import get_folder_path, find_folders, BaseFolderNode
+from .base32hex import b32decode
 from .error import Error
+from .subfolder import get_folder_path, find_folders, BaseFolderNode
 
 
 def get_totp_code(url):
@@ -44,7 +46,7 @@ def get_totp_code(url):
                 if reminder in {2, 4, 5, 7}:
                     padding = '=' * (8 - reminder)
                     secret += padding
-                key = base64.b32decode(secret, casefold=True)
+                key = bytes(b32decode(secret))
                 msg = int(tm).to_bytes(8, byteorder='big')
                 hash = hashlib.__dict__[alg]
                 hm = hmac.new(key, msg=msg, digestmod=hash)
@@ -64,49 +66,93 @@ def get_totp_code(url):
 class Record:
     """Defines a user-friendly Keeper Record for display purposes"""
 
+    @staticmethod
+    def xstr(s):
+        return str(s or '')
+
     def __init__(self, record_uid='', folder='', title='', login='', password='', login_url='', notes='',
                  custom_fields=None, revision=''):
         self.record_uid = record_uid
-        self.folder = folder
-        self.title = title
-        self.login = login
-        self.password = password
-        self.login_url = login_url
-        self.notes = notes
+        self.record_type = ''
+        self.folder = Record.xstr(folder)
+        self.title = Record.xstr(title)
+        self.login = Record.xstr(login)
+        self.password = Record.xstr(password)
+        self.login_url = Record.xstr(login_url)
+        self.notes = Record.xstr(notes)
         self.custom_fields = custom_fields or []  # type: list
         self.attachments = None
         self.revision = revision
         self.unmasked_password = None
         self.totp = None
+        self.version = 2
 
     def load(self, data, **kwargs):
-
-        def xstr(s):
-            return str(s or '')
-
-        if 'folder' in data:
-            self.folder = xstr(data['folder'])
+        self.version = kwargs.get('version', 2)
         if 'title' in data:
-            self.title = xstr(data['title'])
-        if 'secret1' in data:
-            self.login = xstr(data['secret1'])
-        if 'secret2' in data:
-            self.password = xstr(data['secret2'])
+            self.title = Record.xstr(data['title'])
         if 'notes' in data:
-            self.notes = xstr(data['notes'])
-        if 'link' in data:
-            self.login_url = xstr(data['link'])
-        if 'custom' in data:
-            self.custom_fields = data['custom']
+            self.notes = Record.xstr(data['notes'])
+
+        if self.version == 2:
+            if 'secret1' in data:
+                self.login = Record.xstr(data['secret1'])
+            if 'secret2' in data:
+                self.password = Record.xstr(data['secret2'])
+            if 'link' in data:
+                self.login_url = Record.xstr(data['link'])
+            if 'custom' in data:
+                self.custom_fields = data['custom'] or []
+            if 'extra' in kwargs and kwargs['extra']:
+                extra = kwargs['extra']
+                self.attachments = extra.get('files')
+                if 'fields' in extra:
+                    for field in extra['fields']:
+                        if field['field_type'] == 'totp':
+                            self.totp = field['data']
+        elif self.version == 3:
+            self.record_type = data.get('type', 'login')
+            for field in itertools.chain(data['fields'], data.get('custom') or []):
+                field_label = field.get('label', '')
+                field_type = field.get('type', '')
+                field_value = field.get('value', '')
+                if isinstance(field_value, list):
+                    if len(field_value) == 1:
+                        field_value = field_value[0]
+                    elif len(field_value) == 0:
+                        field_value = None
+                if field_value:
+                    if isinstance(field_value, str):
+                        if field_type == 'login' and not self.login:
+                            self.login = field_value
+                            continue
+                        if field_type == 'password' and not self.password:
+                            self.password = field_value
+                            continue
+                        if field_type == 'url' and not self.login_url:
+                            self.login_url = field_value
+                            continue
+                        if field_type == 'oneTimeCode' and not self.totp:
+                            self.totp = field_value
+                            continue
+
+                    if field_type:
+                        field_name = f'{field_type}:{field_label}'
+                    elif field_label:
+                        field_name = field_label
+                    else:
+                        field_name = 'text'
+                    self.append_field_value(field_name, field_value)
+
+        elif self.version == 4:
+            self.record_type = 'file'
+            for field in ['size', 'name', 'type']:
+                if field in data:
+                    self.append_field_value(field, data[field])
+        else:
+            pass
         if 'revision' in kwargs:
             self.revision = kwargs['revision']
-        if 'extra' in kwargs and kwargs['extra']:
-            extra = kwargs['extra']
-            self.attachments = extra.get('files')
-            if 'fields' in extra:
-                for field in extra['fields']:
-                    if field['field_type'] == 'totp':
-                        self.totp = field['data']
 
     def get(self, field):
         result = ''
@@ -116,6 +162,25 @@ class Record:
                 break
         return result
 
+    def append_field_value(self, name, value):
+        if not value:
+            return
+        field = next((x for x in self.custom_fields if x.get('name', '') == name), None)
+        if not field:
+            field = {'type': 'text', 'name': name}
+            self.custom_fields.append(field)
+        field_value = field.get('value', None)
+        if field_value:
+            if not isinstance(field_value, list):
+                field_value = [field_value]
+            if isinstance(value, list):
+                field_value.extend(value)
+            else:
+                field_value.append(value)
+        else:
+            field_value = value
+        field['value'] = field_value
+
     def set_field(self, name, value):
         found = False
         for field in self.custom_fields:
@@ -124,7 +189,7 @@ class Record:
                 found = True
                 break
         if not found:
-            self.custom_fields.append({'name': name, 'value': value})
+            self.custom_fields.append({'type': 'text', 'name': name, 'value': value})
 
     def remove_field(self, name):
         if self.custom_fields:
@@ -151,8 +216,8 @@ class Record:
         if len(self.custom_fields) > 0:
             for c in self.custom_fields:
                 if not 'value' in c: c['value'] = ''
-                if not 'name' in c: c['name'] = ''
-                print('{0:>20s}: {1:<s}'.format(c['name'], c['value']))
+                if not 'name' in c: c['name'] = c['type'] if 'type' in c else ''
+                print('{0:>20s}: {1:<s}'.format(str(c['name']), str(c['value'])))
 
         if self.notes:
             lines = self.notes.split('\n')
@@ -237,6 +302,17 @@ class Record:
                                                                              fol.name))
                                             no = no + 1
 
+        if params and params.breach_watch:
+            bw_status = params.breach_watch.get_record_status(params, self.record_uid)
+            if bw_status and 'status' in bw_status:
+                status = bw_status['status']
+                if status:
+                    if status in {'WEAK', 'BREACHED'}:
+                        status = 'High-Risk Password'
+                    elif status == 'IGNORE':
+                        status = 'Ignored'
+                    print('{0:>20s}: {1:<20s}'.format('Breach Watch', status))
+
         print('')
 
     def mask_password(self):
@@ -280,3 +356,35 @@ class Record:
             'notes': self.notes,
             'custom_fields': self.custom_fields,
         }
+
+    @classmethod
+    def validate_record_data(cls, data, extra, udata):
+        # data - always present (UID, Title, ...)
+        if data:
+            data_types = {
+                'folder': {'field': 'folder', 'name': 'folder', 'type': ''},
+                'title': {'field': 'title', 'name': 'title', 'type': ''},
+                'secret1': {'field': 'secret1', 'name': 'login', 'type': ''},
+                'secret2': {'field': 'secret2', 'name': 'password', 'type': ''},
+                'link': {'field': 'link', 'name': 'url', 'type': ''},
+                'notes': {'field': 'notes', 'name': 'notes', 'type': ''},
+                'custom': {'field': 'custom', 'name': 'custom', 'type': []}
+            }
+
+            for item in data_types:
+                if item in data and type(data.get(item)) != type(data_types[item].get('type')):
+                    raise ValueError('Error validating record data - "' + data_types[item].get('name') + '" is invalid!')
+
+            if 'custom' in data and isinstance(data['custom'], list):
+                invalid = [x for x in data['custom'] if not(x)]
+                if invalid:
+                    raise ValueError('Error validating record data - Invalid custom fields! ' + str(invalid))
+        else:
+            raise Exception('Record is empty!')
+
+        # extra and udata sections are optional
+        if extra:
+            fields = extra.get('fields') or []
+            invalid = [x for x in fields if not(x)]
+            if invalid:
+                raise ValueError('Error validating record extra data - Invalid extra fields! ' + str(invalid))
