@@ -5,32 +5,32 @@
 #              |_|
 #
 # Keeper Commander
-# Copyright 2019 Keeper Security Inc.
+# Copyright 2022 Keeper Security Inc.
 # Contact: ops@keepersecurity.com
 #
 
 import requests
-import base64
 import os
 import json
-import hashlib
-import hmac
 import logging
 
+from typing import Union, Dict
+
 from .params import RestApiContext
 from .error import KeeperApiError, CommunicationError
-from . import APIRequest_pb2 as proto
+from .proto import APIRequest_pb2 as proto
+from . import crypto, utils
+from cryptography.hazmat.primitives.asymmetric import rsa, ec
 
-from Cryptodome.PublicKey import RSA
-from Cryptodome.Cipher import AES, PKCS1_v1_5
+from . import __version__
 
 
 LEGACY_CLIENT_VERSION = 'c14.0.0'
-CLIENT_VERSION = 'c15.0.0'
-
+# CLIENT_VERSION = 'c' + __version__
+CLIENT_VERSION = 'c16.6.2'
 
 SERVER_PUBLIC_KEYS = {
-    1: RSA.importKey(base64.urlsafe_b64decode(
+    1: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEA9Z_CZzxiNUz8-npqI4V10-zW3AL7-M4UQDdd_17759Xzm0MOEfH' +
         'OOsOgZxxNK1DEsbyCTCE05fd3Hz1mn1uGjXvm5HnN2mL_3TOVxyLU6VwH9EDInn' +
         'j4DNMFifs69il3KlviT3llRgPCcjF4xrF8d4SR0_N3eqS1f9CBJPNEKEH-am5Xb' +
@@ -38,7 +38,7 @@ SERVER_PUBLIC_KEYS = {
         'xIXtxRy-4jA49zK-CQrGmWqIm5DzZcBvUtVGZ3UXd6LeMXMJOifvuCneGC2T2uB' +
         '6G2g5yD54-onmKIETyNX0LtpR1MsZmKLgru5ugwIDAQAB')),
 
-    2: RSA.importKey(base64.urlsafe_b64decode(
+    2: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEAkOpym7xC3sSysw5DAidLoVF7JUgnvXejbieDWmEiD-DQOKxzfQq' +
         'YHoFfeeix__bx3wMW3I8cAc8zwZ1JO8hyB2ON732JE2Zp301GAUMnAK_rBhQWmY' +
         'KP_-uXSKeTJPiuaW9PVG0oRJ4MEdS-t1vIA4eDPhI1EexHaY3P2wHKoV8twcGvd' +
@@ -46,7 +46,7 @@ SERVER_PUBLIC_KEYS = {
         'BnQ72Bdbsry0KKVTlcPsudAnnWUtsMJNgmyQbESPm-aVv-GzdVUFvWKpKkAxDpN' +
         'ArPMf0xt8VL2frw2LDe5_n9IMFogUiSYt156_mQIDAQAB')),
 
-    3: RSA.importKey(base64.urlsafe_b64decode(
+    3: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEAyvxCWbLvtMRmq57oFg3mY4DWfkb1dir7b29E8UcwcKDcCsGTqoI' +
         'hubU2pO46TVUXmFgC4E-Zlxt-9F-YA-MY7i_5GrDvySwAy4nbDhRL6Z0kz-rqUi' +
         'rgm9WWsP9v-X_BwzARqq83HNBuzAjf3UHgYDsKmCCarVAzRplZdT3Q5rnNiYPYS' +
@@ -54,7 +54,7 @@ SERVER_PUBLIC_KEYS = {
         'Xn1RdtbNohmMXldAH-C8uIh3Sz8erS4hZFSdUG1WlDsKpyRouNPQ3diorbO88wE' +
         'AgpHjXkOLj63d1fYJBFG0yfu73U80aEZehQkSawIDAQAB')),
 
-    4: RSA.importKey(base64.urlsafe_b64decode(
+    4: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEA0TVoXLpgluaqw3P011zFPSIzWhUMBqXT-Ocjy8NKjJbdrbs53eR' +
         'FKk1waeB3hNn5JEKNVSNbUIe-MjacB9P34iCfKtdnrdDB8JXx0nIbIPzLtcJC4H' +
         'CYASpjX_TVXrU9BgeCE3NUtnIxjHDy8PCbJyAS_Pv299Q_wpLWnkkjq70ZJ2_fX' +
@@ -62,7 +62,7 @@ SERVER_PUBLIC_KEYS = {
         'KMcAbKQcLrml8CMPc2mmf0KQ5MbS_KSbLXHUF-81AsZVHfQRSuigOStQKxgSGL5' +
         'osY4NrEcODbEXtkuDrKNMsZYhijKiUHBj9vvgKwIDAQAB')),
 
-    5: RSA.importKey(base64.urlsafe_b64decode(
+    5: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEAueOWC26w-HlOLW7s88WeWkXpjxK4mkjqngIzwbjnsU9145R51Hv' +
         'sILvjXJNdAuueVDHj3OOtQjfUM6eMMLr-3kaPv68y4FNusvB49uKc5ETI0HtHmH' +
         'FSn9qAZvC7dQHSpYqC2TeCus-xKeUciQ5AmSfwpNtwzM6Oh2TO45zAqSA-QBSk_' +
@@ -70,43 +70,59 @@ SERVER_PUBLIC_KEYS = {
         'hBgTFXRHg5OmJbbPoHj217Yh-kHYA8IWEAHylboH6CVBdrNL4Na0fracQVTm-nO' +
         'WdM95dKk3fH-KJYk_SmwB47ndWACLLi5epLl9vwIDAQAB')),
 
-    6: RSA.importKey(base64.urlsafe_b64decode(
+    6: crypto.load_rsa_public_key(utils.base64_url_decode(
         'MIIBCgKCAQEA2PJRM7-4R97rHwY_zCkFA8B3llawb6gF7oAZCpxprl6KB5z2cqL' +
         'AvUfEOBtnr7RIturX04p3ThnwaFnAR7ADVZWBGOYuAyaLzGHDI5mvs8D-NewG9v' +
         'w8qRkTT7Mb8fuOHC6-_lTp9AF2OA2H4QYiT1vt43KbuD0Y2CCVrOTKzDMXG8msl' +
         '_JvAKt4axY9RGUtBbv0NmpkBCjLZri5AaTMgjLdu8XBXCqoLx7qZL-Bwiv4njw-' +
         'ZAI4jIszJTdGzMtoQ0zL7LBj_TDUBI4Qhf2bZTZlUSL3xeDWOKmd8Frksw3oKyJ' +
         '17oCQK-EGau6EaJRGyasBXl8uOEWmYYgqOWirNwIDAQAB')),
-}
 
+    7: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BK9w6TZFxE6nFNbMfIpULCup2a8xc6w2tUTABjxny7yFmxW0dAEojwC6j6zb5nTlmb1dAx8nwo3qF7RPYGmloRM')),
 
-def encrypt_rsa(data, rsa_key):
-    # type: (bytes, RSA.RsaKey) -> bytes
-    cipher = PKCS1_v1_5.new(rsa_key)
-    return cipher.encrypt(data)
+    8: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BKnhy0obglZJK-igwthNLdknoSXRrGB-mvFRzyb_L-DKKefWjYdFD2888qN1ROczz4n3keYSfKz9Koj90Z6w_tQ')),
+
+    9: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BAsPQdCpLIGXdWNLdAwx-3J5lNqUtKbaOMV56hUj8VzxE2USLHuHHuKDeno0ymJt-acxWV1xPlBfNUShhRTR77g')),
+
+    10: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BNYIh_Sv03nRZUUJveE8d2mxKLIDXv654UbshaItHrCJhd6cT7pdZ_XwbdyxAOCWMkBb9AZ4t1XRCsM8-wkEBRg')),
+
+    11: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BA6uNfeYSvqagwu4TOY6wFK4JyU5C200vJna0lH4PJ-SzGVXej8l9dElyQ58_ljfPs5Rq6zVVXpdDe8A7Y3WRhk')),
+
+    12: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BMjTIlXfohI8TDymsHxo0DqYysCy7yZGJ80WhgOBR4QUd6LBDA6-_318a-jCGW96zxXKMm8clDTKpE8w75KG-FY')),
+
+    13: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BJBDU1P1H21IwIdT2brKkPqbQR0Zl0TIHf7Bz_OO9jaNgIwydMkxt4GpBmkYoprZ_DHUGOrno2faB7pmTR7HhuI')),
+
+    14: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BJFF8j-dH7pDEw_U347w2CBM6xYM8Dk5fPPAktjib-opOqzvvbsER-WDHM4ONCSBf9O_obAHzCyygxmtpktDuiE')),
+
+    15: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BDKyWBvLbyZ-jMueORl3JwJnnEpCiZdN7yUvT0vOyjwpPBCDf6zfL4RWzvSkhAAFnwOni_1tQSl8dfXHbXqXsQ8')),
+
+    16: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BDXyZZnrl0tc2jdC5I61JjwkjK2kr7uet9tZjt8StTiJTAQQmnVOYBgbtP08PWDbecxnHghx3kJ8QXq1XE68y8c')),
+
+    17: crypto.load_ec_public_key(utils.base64_url_decode(
+        'BFX68cb97m9_sweGdOVavFM3j5ot6gveg6xT4BtGahfGhKib-zdZyO9pwvv1cBda9ahkSzo1BQ4NVXp9qRyqVGU')),
+}   # type: Dict[int, Union[rsa.RSAPublicKey, ec.EllipticCurvePublicKey]]
 
 
 def encrypt_aes(data, key):
-    # type: (bytes, bytes) -> bytes
-    iv = os.urandom(12)
-    cipher = AES.new(key=key, mode=AES.MODE_GCM, nonce=iv)
-    enc_data, tag = cipher.encrypt_and_digest(data)
-    return iv + enc_data + tag
+    return crypto.encrypt_aes_v2(data, key)
 
 
 def decrypt_aes(data, key):
-    # type: (bytes, bytes) -> bytes
-    cipher = AES.new(key=key, mode=AES.MODE_GCM, nonce=data[:12])
-    return cipher.decrypt_and_verify(data[12:-16], data[-16:])
+    return crypto.decrypt_aes_v2(data, key)
 
 
-def derive_key_v2(domain, password, salt, iterations):
-    # type: (str, str, bytes, int) -> bytes
-    derived_key = hashlib.pbkdf2_hmac('sha512', (domain+password).encode('utf-8'), salt, iterations, 64)
-    return hmac.new(derived_key, domain.encode('utf-8'), digestmod=hashlib.sha256).digest()
-
-
-def execute_rest(context, endpoint, payload):    # type: (RestApiContext, str, proto.ApiRequestPayload) -> any
+def execute_rest(context, endpoint, payload):
+    # type: (RestApiContext, str, proto.ApiRequestPayload) -> Union[bytes, dict]
     if not context.transmission_key:
         context.transmission_key = os.urandom(32)
 
@@ -118,7 +134,13 @@ def execute_rest(context, endpoint, payload):    # type: (RestApiContext, str, p
         run_request = False
 
         api_request = proto.ApiRequest()
-        api_request.encryptedTransmissionKey = encrypt_rsa(context.transmission_key, SERVER_PUBLIC_KEYS[context.server_key_id])
+        server_public_key = SERVER_PUBLIC_KEYS[context.server_key_id]
+        if isinstance(server_public_key, rsa.RSAPublicKey):
+            api_request.encryptedTransmissionKey = crypto.encrypt_rsa(context.transmission_key, server_public_key)
+        elif isinstance(server_public_key, ec.EllipticCurvePublicKey):
+            api_request.encryptedTransmissionKey = crypto.encrypt_ec(context.transmission_key, server_public_key)
+        else:
+            raise ValueError('Invalid server public key')
         api_request.publicKeyId = context.server_key_id
         api_request.locale = context.locale or 'en_US'
 
@@ -128,14 +150,15 @@ def execute_rest(context, endpoint, payload):    # type: (RestApiContext, str, p
         url = context.server_base + endpoint
 
         logging.debug('>>> Request URL: [%s]', url)
-        rs = requests.post(url, data=request_data, headers={'Content-Type': 'application/octet-stream'})
+        rs = requests.post(url, data=request_data, headers={'Content-Type': 'application/octet-stream'},
+                           proxies=context.proxies, verify=context.certificate_check)
         logging.debug('<<< Response Code: [%d]', rs.status_code)
         logging.debug('<<< Response Headers: [%s]', str(rs.headers))
 
         content_type = rs.headers.get('Content-Type') or ''
         if rs.status_code == 200:
             if content_type == 'application/json':
-                return rs.json()        # type: dict
+                return rs.json()
 
             rs_body = rs.content
             if rs_body:
@@ -257,4 +280,3 @@ def v2_execute(context, rq):
 
         if type(rs_data) is dict:
             raise KeeperApiError(rs_data['error'], rs_data['message'])
-
